"use strict";
/*!
 * Paradigm UI Web
 * Copyright (c) 2017 Miracle Devs, Inc
 * Licensed under MIT (https://gitlab.com/miracledevs-paradigm/ui-web-angularjs/blob/master/LICENSE)
 */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var directive_1 = require("../decorators/directive");
var base_directive_1 = require("./base.directive");
var paradigm_ui_web_shared_1 = require("@miracledevs/paradigm-ui-web-shared");
var $ = require("jquery");
var CommentAreaDirective = /** @class */ (function (_super) {
    tslib_1.__extends(CommentAreaDirective, _super);
    function CommentAreaDirective() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CommentAreaDirective_1 = CommentAreaDirective;
    CommentAreaDirective.prototype.create = function (scope, instanceElement, instanceAttributes, controller, transclude) {
        var _this = this;
        var options = {};
        this.tryGetNumber(options, instanceAttributes, "defaultHeight");
        this.tryGetNumber(options, instanceAttributes, "maxSize");
        this.tryGet(options, instanceAttributes, "maxSizeField");
        this.tryGet(options, instanceAttributes, "maxSizeText");
        this.tryGetBoolean(options, instanceAttributes, "resize");
        this.tryGetBoolean(options, instanceAttributes, "showAlways");
        this.tryGetBoolean(options, instanceAttributes, "restrictEntry");
        this.tryGet(options, instanceAttributes, "negativeClass");
        if (options.restrictEntry) {
            instanceElement.on("keydown", function (event) { return _this.restrictEntry(event, options, instanceElement, false); });
            instanceElement.on("keypress", function (event) { return _this.restrictEntry(event, options, instanceElement, true); });
            instanceElement.on("keyup", function (event) { return _this.restrictEntry(event, options, instanceElement, false); });
            scope.$watch(function () { return scope["ngModel"]; }, function () { return _this.restrictEntry(null, options, instanceElement, false); });
        }
        else {
            instanceElement.on("keypress", function () { return _this.showCharacterLeft(options, instanceElement, true); });
            scope.$watch(function () { return scope["ngModel"]; }, function () { return _this.showCharacterLeft(options, instanceElement, false); });
        }
    };
    CommentAreaDirective.prototype.restrictEntry = function (event, options, element, cancel) {
        if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(options.maxSize)) {
            var value = element.val();
            var length_1 = value.length;
            var available = options.maxSize - length_1;
            if (available < 0)
                available = -1;
            if (available === -1) {
                if (cancel) {
                    event.stopPropagation();
                    event.preventDefault();
                }
                element.val(value.substr(0, options.maxSize));
                available = 0;
            }
            if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(options.maxSizeText)) {
                $(options.maxSizeField).text((available === options.maxSize && !options.showAlways) ? "" : paradigm_ui_web_shared_1.StringExtensions.format(options.maxSizeText, available));
            }
            if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(options.resize) && options.resize) {
                this.checkSize(options, element);
            }
        }
    };
    CommentAreaDirective.prototype.showCharacterLeft = function (options, element, cancel) {
        if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(options.maxSize)) {
            var value = element.val();
            var length_2 = value.length;
            var available = options.maxSize - length_2;
            if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(options.maxSizeText)) {
                var field = $(options.maxSizeField);
                if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(options.negativeClass)) {
                    if (available < 0)
                        field.addClass(options.negativeClass);
                    else
                        field.removeClass(options.negativeClass);
                }
                $(options.maxSizeField).text((available === options.maxSize && !options.showAlways) ? "" : paradigm_ui_web_shared_1.StringExtensions.format(options.maxSizeText, available));
            }
            if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(options.resize) && options.resize) {
                this.checkSize(options, element);
            }
        }
        else {
            if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(options.resize) && options.resize) {
                this.checkSize(options, element);
            }
        }
    };
    CommentAreaDirective.prototype.checkSize = function (options, element) {
        var control = element[0];
        var defaultHeight = options.defaultHeight || 0;
        control.style.height = "1px";
        if (control.scrollHeight > control.clientHeight) {
            if (control.scrollHeight > defaultHeight)
                control.style.height = control.scrollHeight + "px";
            else
                control.style.height = defaultHeight + "px";
        }
        else if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(options.defaultHeight)) {
            control.style.height = defaultHeight + "px";
        }
    };
    CommentAreaDirective.factory = function () {
        return new CommentAreaDirective_1();
    };
    CommentAreaDirective = CommentAreaDirective_1 = tslib_1.__decorate([
        directive_1.Directive({
            name: "commentArea",
            restrict: "A",
            require: "ngModel",
            bindings: {
                ngModel: "="
            }
        })
    ], CommentAreaDirective);
    return CommentAreaDirective;
    var CommentAreaDirective_1;
}(base_directive_1.DirectiveBase));
exports.CommentAreaDirective = CommentAreaDirective;
