"use strict";
/*!
 * Paradigm UI Web
 * Copyright (c) 2017 Miracle Devs, Inc
 * Licensed under MIT (https://gitlab.com/miracledevs-paradigm/ui-web-angularjs/blob/master/LICENSE)
 */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var base_directive_1 = require("./base.directive");
var directive_1 = require("../decorators/directive");
var angular_service_1 = require("../services/angular.service");
var paradigm_ui_web_shared_1 = require("@miracledevs/paradigm-ui-web-shared");
var $ = require("jquery");
var HorizontalScrollerDirective = /** @class */ (function (_super) {
    tslib_1.__extends(HorizontalScrollerDirective, _super);
    function HorizontalScrollerDirective(interpolate) {
        var _this = _super.call(this) || this;
        _this.interpolate = interpolate;
        return _this;
    }
    HorizontalScrollerDirective_1 = HorizontalScrollerDirective;
    HorizontalScrollerDirective.prototype.create = function (scope, instanceElement, instanceAttributes, controller, transclude) {
        var options = {};
        options.element = instanceElement;
        this.tryGet(options, instanceAttributes, "container");
        this.tryGet(options, instanceAttributes, "content");
        this.tryGet(options, instanceAttributes, "leftArrow");
        this.tryGet(options, instanceAttributes, "rightArrow");
        this.tryGetNumber(options, instanceAttributes, "speed");
        this.tryGetNumber(options, instanceAttributes, "friction");
        this.tryGetNumber(options, instanceAttributes, "fps");
        this.tryGetNumber(options, instanceAttributes, "minVelocity");
        if (paradigm_ui_web_shared_1.StringExtensions.isNullOrWhiteSpace(options.container) ||
            paradigm_ui_web_shared_1.StringExtensions.isNullOrWhiteSpace(options.content) ||
            paradigm_ui_web_shared_1.StringExtensions.isNullOrWhiteSpace(options.leftArrow) ||
            paradigm_ui_web_shared_1.StringExtensions.isNullOrWhiteSpace(options.rightArrow))
            return;
        instanceElement[0]["scrollerInstance"] = new HorizontalScrollerInstance(options);
        scope.$watch(function () { return instanceElement[0].innerHTML; }, function () { return instanceElement[0]["scrollerInstance"].enableScroll(); });
    };
    HorizontalScrollerDirective.prototype.dispose = function (scope, instanceElement, instanceAttributes, controller, transclude) {
        instanceElement[0]["scrollerInstance"].dispose();
        _super.prototype.dispose.call(this, scope, instanceElement, instanceAttributes, controller, transclude);
    };
    HorizontalScrollerDirective.factory = function (interpolate) {
        return new HorizontalScrollerDirective_1(interpolate);
    };
    HorizontalScrollerDirective = HorizontalScrollerDirective_1 = tslib_1.__decorate([
        directive_1.Directive({
            name: "horizontalScroller",
            restrict: "A",
            dependencies: [angular_service_1.AngularServices.interpolate]
        })
    ], HorizontalScrollerDirective);
    return HorizontalScrollerDirective;
    var HorizontalScrollerDirective_1;
}(base_directive_1.DirectiveBase));
exports.HorizontalScrollerDirective = HorizontalScrollerDirective;
var HorizontalScrollerInstance = /** @class */ (function () {
    function HorizontalScrollerInstance(options) {
        var _this = this;
        this.element = $(options.element);
        this.container = this.element.find(options.container);
        this.content = this.element.find(options.content);
        this.leftArrow = this.element.find(options.leftArrow);
        this.rightArrow = this.element.find(options.rightArrow);
        this.position = 0;
        this.velocity = 0;
        this.speed = options.speed || 140;
        this.friction = options.friction || 0.95;
        this.millisecondsPerFrame = 1000 / (options.fps || 60);
        this.minVelocity = options.minVelocity || 10;
        this.enableScroll();
        $(window).on("resize", function () { return _this.enableScroll(); });
        this.leftArrow.on("mousedown touchstart", function () { return _this.moveLeft(); });
        this.leftArrow.on("mouseup mouseleave touchend touchcancel", function () { return _this.pressing = false; });
        this.rightArrow.on("mousedown touchstart", function () { return _this.moveRight(); });
        this.rightArrow.on("mouseup mouseleave touchend touchcancel", function () { return _this.pressing = false; });
    }
    HorizontalScrollerInstance.prototype.enableScroll = function () {
        var wContainer = this.container.width();
        var wContent = this.content.width();
        if (wContent < wContainer) {
            this.leftArrow.css("display", "none");
            this.rightArrow.css("display", "none");
            this.position = 0;
        }
        else {
            this.leftArrow.css("display", "inline-block");
            this.rightArrow.css("display", "inline-block");
        }
        this.checkConstraints();
        this.applyPosition();
    };
    HorizontalScrollerInstance.prototype.applyPosition = function () {
        var translate = "translate(" + this.position + "px, 0)";
        var translate3D = "translate3d(" + this.position + "px, 0, 0)";
        this.content.css({
            '-ms-transform': translate,
            '-moz-transform': translate3D,
            '-webkit-transform': translate3D,
            'transform': translate3D
        });
    };
    HorizontalScrollerInstance.prototype.checkConstraints = function () {
        var wContainer = this.container.width();
        var wContent = this.content.width();
        var minMovement = wContainer - wContent;
        if (wContent <= wContainer || this.position >= 0)
            this.position = 0;
        if (wContent >= wContainer && this.position <= minMovement)
            this.position = minMovement;
    };
    HorizontalScrollerInstance.prototype.killInterval = function () {
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(this.intervalId))
            return;
        window.clearInterval(this.intervalId);
    };
    HorizontalScrollerInstance.prototype.getMilliseconds = function () {
        return (new Date()).getTime();
    };
    HorizontalScrollerInstance.prototype.moveLeft = function () {
        var _this = this;
        this.pressing = true;
        this.direction = 1;
        this.lastTime = this.getMilliseconds();
        this.killInterval();
        this.intervalId = window.setInterval(function () { return _this.move(); }, this.millisecondsPerFrame);
    };
    HorizontalScrollerInstance.prototype.moveRight = function () {
        var _this = this;
        this.pressing = true;
        this.direction = -1;
        this.lastTime = this.getMilliseconds();
        this.killInterval();
        this.intervalId = window.setInterval(function () { return _this.move(); }, this.millisecondsPerFrame);
    };
    HorizontalScrollerInstance.prototype.move = function () {
        var deltaTime = (this.getMilliseconds() - this.lastTime) / 1000;
        if (this.pressing) {
            this.velocity = this.speed;
        }
        this.velocity *= this.friction;
        this.position = this.position + (this.direction * this.velocity * deltaTime);
        this.checkConstraints();
        this.applyPosition();
        if (this.velocity <= this.minVelocity) {
            this.killInterval();
        }
        this.lastTime = this.getMilliseconds();
    };
    HorizontalScrollerInstance.prototype.dispose = function () {
        this.container.remove();
        this.content.remove();
        this.leftArrow.remove();
        this.rightArrow.remove();
    };
    return HorizontalScrollerInstance;
}());
exports.HorizontalScrollerInstance = HorizontalScrollerInstance;
